<?php

namespace Drupal\entity_hierarchy\Plugin\views\argument;

use Drupal\entity_hierarchy\Storage\Record;
use Drupal\views\Plugin\views\query\Sql;

/**
 * Argument to limit to children of an entity.
 *
 * @ingroup views_argument_handlers
 *
 * @ViewsArgument("entity_hierarchy_argument_is_child_of_entity")
 */
class HierarchyIsChildOfEntity extends EntityHierarchyArgumentPluginBase {

  /**
   * Set up the query for this argument.
   *
   * The argument sent may be found at $this->argument.
   */
  public function query($group_by = FALSE) {
    $this->ensureMyTable();
    // Load the actual entity.
    $filtered = FALSE;
    if ($entity = $this->loadEntity()) {
      $queryBuilder = $this->queryBuilderFactory->get($this->configuration['entity_hierarchy_field'], $entity->getEntityTypeId());
      $children = $queryBuilder->findDescendants($entity, $this->options['depth'])->map(fn (Record $node) => $node->getId());
      assert($this->query instanceof Sql);
      if (!$children) {
        $this->query->addWhereExpression(0, '1 <> 1');
        return;
      }
      $this->query->addWhere(0, "$this->tableAlias.$this->realField", $children, 'IN');
      $filtered = TRUE;
    }
    // The parent entity doesn't exist, or isn't in the tree and hence has no
    // children.
    if (!$filtered) {
      // Add a killswitch.
      $this->query->addWhereExpression(0, '1 <> 1');
    }
  }

}
