<?php

declare(strict_types=1);

namespace Drupal\entity_hierarchy\Plugin\views\sort;

use Drupal\Core\Database\Connection;
use Drupal\Core\Entity\EntityFieldManagerInterface;
use Drupal\Core\Entity\EntityTypeManagerInterface;
use Drupal\entity_hierarchy\Query\TreeOrderSelect;
use Drupal\views\Plugin\views\query\Sql;
use Drupal\views\Plugin\views\sort\SortPluginBase;
use Drupal\views\Views;
use Symfony\Component\DependencyInjection\ContainerInterface;

/**
 * Defines a class for sorting by depth.
 *
 * @ingroup views_sort_handlers
 *
 * @ViewsSort("entity_hierarchy_order")
 */
final class TreeOrder extends SortPluginBase {

  protected EntityFieldManagerInterface $entityFieldManager;
  protected EntityTypeManagerInterface $entityTypeManager;
  protected Connection $database;

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container, array $configuration, $plugin_id, $plugin_definition): static {
    $instance = parent::create($container, $configuration, $plugin_id, $plugin_definition);
    $instance->entityTypeManager = $container->get('entity_type.manager');
    $instance->entityFieldManager = $container->get('entity_field.manager');
    $instance->database = $container->get('database');
    return $instance;
  }

  /**
   * {@inheritdoc}
   */
  public function query() {
    $this->ensureMyTable();
    assert($this->query instanceof Sql);
    $query = new TreeOrderSelect($this->database, $this->entityFieldManager->getFieldStorageDefinitions($this->getEntityType())[$this->configuration['entity_hierarchy_field']], $this->entityTypeManager);

    $def = $this->definition;
    $def['type'] = 'LEFT';
    $def['table formula'] = $query;
    $def['left_table'] = $this->tableAlias;
    $def['field'] = 'id';
    $def['adjusted'] = TRUE;

    $join = Views::pluginManager('join')->createInstance('standard', $def);
    $alias = 'depth_for_' . $this->table;
    $joinAlias = $this->query->addRelationship($alias, $join, $this->definition['base'], $this->relationship);
    $this->query->addOrderBy($joinAlias, 'ordinal');
  }

}
