<?php

declare(strict_types=1);

namespace Drupal\Tests\entity_hierarchy\Functional;

use Drupal\Core\Entity\Sql\SqlContentEntityStorage;
use Drupal\field\Entity\FieldConfig;
use Drupal\FunctionalTests\Update\UpdatePathTestBase;

/**
 * Defines a class for testing update path.
 *
 * @group entity_hierarchy
 */
final class UpdateFieldSchemaTest extends UpdatePathTestBase {

  /**
   * {@inheritdoc}
   */
  protected function setDatabaseDumpFiles(): void {
    $this->databaseDumpFiles = [
      dirname(__DIR__, 2) . '/fixtures/3284026-dump.sql.gz',
    ];
  }

  /**
   * Tests update path.
   */
  public function testUpdatePath(): void {
    $field = FieldConfig::load('node.page.field_parent');
    $this->assertNotNull($field);
    $node_storage = \Drupal::entityTypeManager()->getStorage('node');
    assert($node_storage instanceof SqlContentEntityStorage);
    $storage_definition = $field->getFieldStorageDefinition();
    $mapping = $node_storage->getTableMapping();
    $column_name = $mapping->getFieldColumnName($storage_definition, 'weight');
    $table_name = $mapping->getFieldTableName('field_parent');
    $this->assertTrue(\Drupal::database()->schema()->fieldExists($table_name, $column_name));
    $schema = \Drupal::keyValue('entity.storage_schema.sql')->get($storage_definition->getTargetEntityTypeId() . '.field_schema_data.' . $storage_definition->getName(), []);
    $this->assertFalse($schema[$table_name]['fields'][$column_name]['not null']);
    $this->assertEquals('int', $schema[$table_name]['fields'][$column_name]['type']);
    $this->assertArrayNotHasKey('size', $schema[$table_name]['fields'][$column_name]);
    $this->runUpdates();
    $this->assertTrue(\Drupal::database()->schema()->fieldExists($table_name, $column_name));
    $schema = \Drupal::keyValue('entity.storage_schema.sql')->get($storage_definition->getTargetEntityTypeId() . '.field_schema_data.' . $storage_definition->getName(), []);
    $this->assertTrue($schema[$table_name]['fields'][$column_name]['not null']);
    $this->assertEquals('int', $schema[$table_name]['fields'][$column_name]['type']);
    $this->assertEquals('small', $schema[$table_name]['fields'][$column_name]['size']);
  }

}
